@@ -118,17 +118,25 @@ test_that("corrupt tokens object does not crash R", {
 
 test_that("tokens_recompile: flag use of padding even when it does not reindex tokens", {
     
-    toks <- list(0:26) # has padding, but no gap
-    attr(toks, 'types') <- letters # no duplication
-    attr(toks, 'class') <- 'tokens'
+    toks <- quanteda:::build_tokens(
+        list(0:26), # has padding, but no gap
+        letters,
+        docvars = quanteda:::make_docvars(1L)
+    )
     expect_true(attr(quanteda:::tokens_recompile(toks, 'C++'), 'padding'))
     
 })
 
 test_that("non-ascii types are UTF8 encoded", {
-    toks <- list(c(1:3))
-    toks <- quanteda:::cpp_tokens_recompile(toks, c('あ', 'い', 'う', 'え', 'お'))
-    expect_equal(Encoding(attr(toks, 'types')), rep('UTF-8', 3))
+    
+    toks <- quanteda:::build_tokens(
+        list(c(1, 2, 3)),
+        c('あ', 'い', 'う', 'え', 'お'),
+        docvars = quanteda:::make_docvars(1L)
+    )
+    
+    toks2 <- quanteda:::tokens_recompile(toks, 'C++')
+    expect_equal(Encoding(attr(toks2, 'types')), rep('UTF-8', 3))
 })
 
 test_that("keep gap and dupli argument works, #1278", {
@@ -139,33 +147,12 @@ test_that("keep gap and dupli argument works, #1278", {
         docvars = quanteda:::make_docvars(1L)
     )
 
-    toks2 <- quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = TRUE)
+    toks2 <- quanteda:::tokens_recompile(toks, 'C++')
     expect_equal(attr(toks2, 'padding'), TRUE)
     expect_equal(attr(toks2, 'types'), c("b", "c"))
     
-    toks3 <- quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = FALSE)
-    expect_equal(attr(toks3, 'padding'), TRUE)
-    expect_equal(attr(toks3, 'types'), c("b", "c", "c"))
-    
-    toks4 <- quanteda:::tokens_recompile(toks, 'C++', gap = FALSE, dup = TRUE)
-    expect_equal(attr(toks4, 'padding'), TRUE)
-    expect_equal(attr(toks4, 'types'), c("a", "b", "c", "d"))
-    
-    toks5 <- quanteda:::tokens_recompile(toks, 'C++', gap = FALSE, dup = FALSE)
-    expect_equal(attr(toks5, 'padding'), TRUE)
-    expect_equal(attr(toks5, 'types'), c("a", "b", "c", "c", "d"))
-    
-    expect_equal(quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = TRUE),
-                 quanteda:::tokens_recompile(toks, 'R', gap = TRUE, dup = TRUE))
-
-    expect_equal(quanteda:::tokens_recompile(toks, 'C++', gap = FALSE, dup = TRUE),
-                 quanteda:::tokens_recompile(toks, 'R', gap = FALSE, dup = TRUE))
-    
-    expect_equal(quanteda:::tokens_recompile(toks, 'C++', gap = TRUE, dup = FALSE),
-                 quanteda:::tokens_recompile(toks, 'R', gap = TRUE, dup = FALSE))
-    
-    expect_equal(quanteda:::tokens_recompile(toks, 'C++', gap = FALSE, dup = FALSE),
-                 quanteda:::tokens_recompile(toks, 'R', gap = FALSE, dup = FALSE))
+    expect_equal(quanteda:::tokens_recompile(toks, 'C++'),
+                 quanteda:::tokens_recompile(toks, 'R'))
     
     toks_pad <- quanteda:::build_tokens(
         list(c(0, 2, 3, 4)),
@@ -174,33 +161,23 @@ test_that("keep gap and dupli argument works, #1278", {
         docvars = quanteda:::make_docvars(1L)
     )
     
-    toks_pad2 <- quanteda:::tokens_recompile(toks_pad, 'C++', gap = TRUE, dup = TRUE)
+    toks_pad2 <- quanteda:::tokens_recompile(toks_pad, 'C++')
     expect_equal(attr(toks_pad2, 'padding'), TRUE)
     expect_equal(attr(toks_pad2, 'types'), c("b", "c"))
     
-    toks_pad3 <- quanteda:::tokens_recompile(toks_pad, 'C++', gap = TRUE, dup = FALSE)
-    expect_equal(attr(toks_pad3, 'padding'), TRUE)
-    expect_equal(attr(toks_pad3, 'types'), c("b", "c", "c"))
+    expect_equal(quanteda:::tokens_recompile(toks_pad, 'C++'),
+                 quanteda:::tokens_recompile(toks_pad, 'R'))
     
-    toks_pad4 <- quanteda:::tokens_recompile(toks_pad, 'C++', gap = FALSE, dup = TRUE)
-    expect_equal(attr(toks_pad4, 'padding'), TRUE)
-    expect_equal(attr(toks_pad4, 'types'), c("a", "b", "c", "d"))
+    toks_err <- quanteda:::build_tokens(
+        list(c(2, 3, 4, 6)),
+        c('a', 'b', 'c', 'c', 'd'),
+        padding = TRUE,
+        docvars = quanteda:::make_docvars(1L)
+    )
+    expect_error(
+        quanteda:::tokens_recompile(toks_err, 'C++')
+    )
     
-    toks_pad5 <- quanteda:::tokens_recompile(toks_pad, 'C++', gap = FALSE, dup = FALSE)
-    expect_equal(attr(toks_pad5, 'padding'), TRUE)
-    expect_equal(attr(toks_pad5, 'types'), c("a", "b", "c", "c", "d"))
-    
-    expect_equal(quanteda:::tokens_recompile(toks_pad, 'C++', gap = TRUE, dup = TRUE),
-                 quanteda:::tokens_recompile(toks_pad, 'R', gap = TRUE, dup = TRUE))
-    
-    expect_equal(quanteda:::tokens_recompile(toks_pad, 'C++', gap = FALSE, dup = TRUE),
-                 quanteda:::tokens_recompile(toks_pad, 'R', gap = FALSE, dup = TRUE))
-    
-    expect_equal(quanteda:::tokens_recompile(toks_pad, 'C++', gap = TRUE, dup = FALSE),
-                 quanteda:::tokens_recompile(toks_pad, 'R', gap = TRUE, dup = FALSE))
-    
-    expect_equal(quanteda:::tokens_recompile(toks_pad, 'C++', gap = FALSE, dup = FALSE),
-                 quanteda:::tokens_recompile(toks_pad, 'R', gap = FALSE, dup = FALSE))
 })
 
 test_that("set encoding when no gap or duplication is found, #1387", {
